@@ -8,7 +8,7 @@ dtiTensor <- function(object,  ...) cat("No DTI tensor calculation defined for t
 
 setGeneric("dtiTensor", function(object,  ...) standardGeneric("dtiTensor"))
 
-setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="replicates",varmodel="local",mc.cores=getOption("mc.cores", 2L)) {
+setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="replicates",varmodel="local",mc.cores=getOption("mc.cores", 1L)) {
 #  available methods are 
 #  "linear" - use linearized model (log-transformed)
 #  "nonlinear" - use nonlinear model with parametrization according to Koay et.al. (2006)
@@ -28,39 +28,31 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
     cat("No parameters for model of error standard deviation found\n estimating these parameters\n You may prefer to run sdpar before calling dtiTensor")
     sdcoef <- sdpar(object,interactive=FALSE)@sdcoef
   }
-  z <- .Fortran("outlier",
-                as.double(object@si),
-                as.integer(prod(ddim)),
-                as.integer(ngrad),
-                as.logical((1:ngrad)%in%s0ind),
-                as.integer(ns0),
-                si=integer(prod(ddim)*ngrad),
-                index=integer(prod(ddim)),
-                lindex=integer(1),
-                DUP=FALSE,
-                PACKAGE="dti")[c("si","index","lindex")]
-  si <- array(z$si,c(ddim,ngrad))
-  index <- if(z$lindex>0) z$index[1:z$lindex] else numeric(0)
+  z <- sioutlier(object@si,s0ind,mc.cores=mc.cores)
+#
+#  this does not scale well with openMP
+#
+  si <- array(z$si,c(ngrad,ddim))
+  index <- z$index
   rm(z)
   gc()
   if(method=="linear"){
      ngrad0 <- ngrad - length(s0ind)
-     s0 <- si[,,,s0ind]
-     si <- si[,,,-s0ind]
+     s0 <- si[s0ind,,,]
+     si <- si[-s0ind,,,]
      if(ns0>1) {
-         dim(s0) <- c(prod(ddim),ns0)
-         s0 <- s0 %*% rep(1/ns0,ns0)
+         dim(s0) <- c(ns0,prod(ddim))
+         s0 <- rep(1/ns0,ns0)%*%s0
          dim(s0) <- ddim
      }
      mask <- s0 > object@level
      mask <- connect.mask(mask)
-     dim(s0) <- dim(si) <- NULL
-     ttt <- -log(si/s0)
+     idsi <- 1:length(dim(si))
+     ttt <- -log(sweep(si,idsi[-1],s0,"/"))
      ttt[is.na(ttt)] <- 0
      ttt[(ttt == Inf)] <- 0
      ttt[(ttt == -Inf)] <- 0
-     dim(ttt) <- c(prod(ddim),ngrad0)
-     ttt <- t(ttt)
+     dim(ttt) <- c(ngrad0,prod(ddim))
      cat("Data transformation completed ",format(Sys.time()),"\n")
 
      btbsvd <- svd(object@btb[,-s0ind])
@@ -75,24 +67,7 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
      sigma2 <- rss/(ngrad0-6)
      D[c(1,4,6),!mask] <- 1e-6
      D[c(2,3,5),!mask] <- 0
-#  replace non-tensors (with negative eigenvalues) by a small isotropic tensor 
-#      ind <- array(.Fortran("dti3Dev",
-#                           as.double(D),
-#                           as.integer(nvox),
-#                           as.logical(mask),
-#                           ev=double(3*prod(ddim)),
-#                           DUP=FALSE,
-#                           PACKAGE="dti")$ev,c(3,ddim))[1,,,]<1e-6
-#       if(sum(ind&mask)>0){
-#           D[c(1,4,6),ind&mask] <- 1e-6
-#           D[c(2,3,5),ind&mask] <- 0
-#       }
-#  Replace tensors with negative eigenvalues
-     D <- .Fortran("dti3Dreg",
-                   D=as.double(D),
-                   as.integer(prod(ddim)),
-                   DUP=FALSE,
-                   PACKAGE="dti")$D                   
+     D <- dti3Dreg(D,mc.cores=mc.cores)
      dim(D) <- c(6,ddim)
      dim(res) <- c(ngrad0,ddim)
      cat("Variance estimates generated ",format(Sys.time()),"\n")
@@ -102,7 +77,6 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
   } else {
 #  method == "nonlinear" 
      ngrad0 <- ngrad
-     si <- aperm(si,c(4,1:3))
      s0 <- si[s0ind,,,]
      if(ns0>1) {
          dim(s0) <- c(ns0,nvox)
@@ -112,8 +86,8 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
      mask <- s0 > object@level
      mask <- connect.mask(mask)
      df <- sum(table(object@replind)-1)
-     cat("start nonlinear regression",format(Sys.time()),"\n")
      if(mc.cores==1){
+     cat("start nonlinear regression",format(Sys.time()),"\n")
      z <- .Fortran("nlrdtirg",
                 as.integer(si),
                 as.integer(ngrad),
@@ -134,10 +108,11 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
      rss <- z$rss
      th0 <- z$th0
      } else {
-        dim(si) <- c(dim(si)[1],nvox)
+     cat("start nonlinear regression using ",mc.cores, "cores", format(Sys.time()),"\n")
         z <- matrix(0,8+ngrad,nvox)
-        z[,mask] <- pmatrix(si[,mask],pnlrdtirg,btb=object@btb,sdcoef=sdcoef,s0ind=s0ind,
-                     mc.cores=mc.cores,mc.silent = TRUE)
+        dim(si) <- c(ngrad,nvox)
+        z[,mask] <- plmatrix(si[,mask],pnlrdtirg,btb=object@btb,sdcoef=sdcoef,s0ind=s0ind,
+                     ngrad = ngrad,mc.cores=mc.cores)
         th0 <- z[1,]
         D <- z[2:7,]
         rss <- z[8,]
@@ -153,12 +128,12 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
      indD <- (1:n)[D[2,,,, drop=FALSE]==0&D[3,,,, drop=FALSE]==0&D[5,,,, drop=FALSE]==0&mask]
      dim(mask) <- ddim
 # this does not work in case of 2D data: 
-#     indD <- (1:n)[D[2,,,]==0&D[3,,,]==0&D[5,,,]==0&mask]
      if(length(indD)>0){
+     cat("length of IndD",length(indD),"\n")
      dim(si) <- c(ngrad,n)
      dim(D) <- c(6,n)
      dim(res) <- c(ngrad,n)
-#     cat("length of IndD",length(indD),"\n")
+     if(mc.cores==1||length(indD)<2*mc.cores){
      for(i in indD){
         zz <- optim(c(1,0,0,1,0,1),opttensR,method="BFGS",si=si[-s0ind,i],s0=s0[i],grad=grad[,-s0ind],sdcoef=sdcoef)
         D[,i] <- rho2D(zz$par)
@@ -167,6 +142,15 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
         res[s0ind,i] <- 0
         res[-s0ind,i] <- tensRres(zz$par,si[-s0ind,i],s0[i],grad[,-s0ind])
      }
+     } else {
+        zz <- pmatrix(si[,indD],pnltens,grad=grad[,-s0ind],s0ind=s0ind,
+                      sdcoef=sdcoef,mc.cores=min(mc.cores,length(indD)))
+        D[,indD] <- zz[1:6,]
+        th0[indD] <- zz[7,]
+        rss[indD] <- zz[8,]
+        res[s0ind,indD] <- 0
+        res[-s0ind,indD] <- zz[-(1:8),]
+     }
      dim(D) <- c(6,ddim)
      dim(res) <- c(ngrad,ddim)
      }
@@ -175,44 +159,11 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
      rm(z)
      gc()
   }
-  lags <- c(5,5,3)
-  scorr <- .Fortran("mcorr",as.double(res),
-                   as.logical(mask),
-                   as.integer(ddim[1]),
-                   as.integer(ddim[2]),
-                   as.integer(ddim[3]),
-                   as.integer(ngrad0),
-                   double(prod(ddim)),
-                   double(prod(ddim)),
-                   scorr = double(prod(lags)),
-                   as.integer(lags[1]),
-                   as.integer(lags[2]),
-                   as.integer(lags[3]),
-                   PACKAGE="dti",DUP=FALSE)$scorr
-  dim(scorr) <- lags
-  scorr[is.na(scorr)] <- 0
-  cat("estimated spatial correlations",format(Sys.time()),"\n")
-  cat("first order  correlation in x-direction",signif(scorr[2,1,1],3),"\n")
-  cat("first order  correlation in y-direction",signif(scorr[1,2,1],3),"\n")
-  cat("first order  correlation in z-direction",signif(scorr[1,1,2],3),"\n")
-
-  scorr[is.na(scorr)] <- 0
-  bw <- optim(c(2,2,2),corrrisk,method="L-BFGS-B",lower=c(.2,.2,.2),
-  upper=c(3,3,3),lag=lags,data=scorr)$par
-  bw[bw <= .25] <- 0
-  cat("estimated corresponding bandwidths",format(Sys.time()),"\n")
-  if(mc.cores<=1) ev <- .Fortran("dti3Dev",
-                       as.double(D),
-                       as.integer(nvox),
-                       as.logical(mask),
-                       ev=double(3*nvox),
-                       DUP=FALSE,
-                       PACKAGE="dti")$ev
-   else {
-      ev <- matrix(0,3,prod(ddim))
-      dim(D) <- c(6,prod(ddim))
-      ev[,mask] <- pmatrix(D[,mask],pdti3Dev,mc.cores=mc.cores,mc.silent = TRUE)
-   }
+#
+#   get spatial correlation
+#
+  scorr <- mcorr(res,mask,ddim,ngrad0,lags=c(5,5,3),mc.cores=mc.cores)
+  ev <- dti3Dev(D,mask,mc.cores=mc.cores)
   dim(ev) <- c(3,ddim)   
   dim(D) <- c(6,ddim)   
   scale <- quantile(ev[3,,,][mask],.95)
@@ -222,11 +173,12 @@ setMethod("dtiTensor","dtiData",function(object, method="nonlinear",varmethod="r
                 D     = D,
                 th0   = th0,
                 sigma = sigma2,
-                scorr = scorr, 
-                bw = bw, 
+                scorr = scorr$scorr, 
+                bw = scorr$bw, 
                 mask = mask,
                 hmax = 1,
                 gradient = object@gradient,
+                bvalue = object@bvalue,
                 btb   = object@btb,
                 ngrad = object@ngrad, # = dim(btb)[2]
                 s0ind = object@s0ind,
@@ -296,27 +248,7 @@ function(object, which, mc.cores=getOption("mc.cores", 2L)) {
   args <- c(object@call,args)
   ddim <- object@ddim
   n <- prod(ddim)
-  z <- if(mc.cores==1) .Fortran("dtiind3D",
-                as.double(object@D),
-                as.integer(n),
-                as.logical(object@mask),
-                fa=double(n),
-                ga=double(n),
-                md=double(n),
-                andir=double(3*n),
-                bary=double(3*n),
-                DUP=FALSE,
-                PACKAGE="dti")[c("fa","ga","md","andir","bary")] else {
-           D <- matrix(object@D,6,prod(ddim))[,object@mask]
-           res <- matrix(0,9,prod(ddim))
-           res[,object@mask] <- pmatrix(D,pdtiind3D,mc.cores=mc.cores,mc.silent = TRUE)
-           list(andir=res[1:3,],
-                   fa=res[4,],
-                   ga=res[5,],
-                   md=res[6,],
-                 bary=res[7:9,])
-                }
-
+  z <- dtiind3D(object@D,object@mask,mc.cores=mc.cores)
   invisible(new("dtiIndices",
                 call = args,
                 fa = array(z$fa,ddim),
@@ -325,6 +257,7 @@ function(object, which, mc.cores=getOption("mc.cores", 2L)) {
                 andir = array(z$andir,c(3,ddim)),
                 bary = array(z$bary,c(3,ddim)),
                 gradient = object@gradient,
+                bvalue = object@bvalue,
                 btb   = object@btb,
                 ngrad = object@ngrad, # = dim(btb)[2]
                 s0ind = object@s0ind,
